--[[	Config	]]
local ShowMainZoneText	= true
local ShowSubZoneText	= true
local ShowCoords		= true
local PreciseCoords		= false -- coords up to ONE decimal places
local MorePrecise		= false -- coords up to TWO decimal places (no need to check 'PreciseCoords' if 'MorePrecise' is true)
--[[	Config end	]]

local locale = GetLocale()
local L = locale == 'zhCN' and {
	['|cff00ff00Shift + Click|r insert zone text\ninto chat edit box'] = '|cff00ff00Shift + 点击|r 插入地区文字到聊天输入框',
	['|cff00ff00Left-Click|r toggle worldmap'] = '|cff00ff00左键点击|r 打开地图',
	['|cff00ff00Right-Click|r toggle Atlas'] = '|cff00ff00右键点击|r 打开Atlas',
	['Zone'] = '地区',
	['Subzone'] = '子地区',
	['Coordinates'] = '坐标',
} or locale == 'zhTW' and {
	['|cff00ff00Shift + Click|r insert zone text\ninto chat edit box'] = '|cff00ff00Shift + 點擊|r 插入地區文字到聊天輸入框',
	['|cff00ff00Left-Click|r toggle worldmap'] = '|cff00ff00左鍵點擊|r 打開地圖',
	['|cff00ff00Right-Click|r toggle Atlas'] = '|cff00ff00右鍵點擊|r 打開Atlas',
	['Zone'] = '地區',
	['Subzone'] = '子地區',
	['Coordinates'] = '坐標',
} or {
	['|cff00ff00Shift + Click|r insert zone text\ninto chat edit box'] = '|cff00ff00Shift + Click|r insert zone text\ninto chat edit box',
	['|cff00ff00Left-Click|r toggle worldmap'] = '|cff00ff00Left-Click|r toggle worldmap',
	['|cff00ff00Right-Click|r toggle Atlas'] = '|cff00ff00Right-Click|r toggle Atlas',
	['Zone'] = 'Zone',
	['Subzone'] = 'Subzone',
	['Coordinates'] = 'Coordinates',
}


local obj = LibStub('LibDataBroker-1.1'):NewDataObject('leafZone', {
	type = 'data source',
	icon = 'Interface\\WorldMap\\WorldMap-Icon',
	text = '',
})

local text_zone = ''
local text_color = 'ffff00'
local status_ininstance = false
local f = CreateFrame('Frame')
local elapsed = 0

local badinstanceType = {
	['arena'] = true,
	['party'] = true,
	['raid'] = true,
}

local CoordsTextFormat = (ShowMainZoneText or ShowSubZoneText) and ' (%.0f, %.0f)' or '%.0f, %.0f'
if MorePrecise then CoordsTextFormat = CoordsTextFormat:gsub('%%%.0f', '%%%.2f')
elseif PreciseCoords then CoordsTextFormat = CoordsTextFormat:gsub('%%%.0f', '%%%.1f') end

local function GetCoords()
	if status_ininstance or (not ShowCoords) then return '' end
	local x, y = GetPlayerMapPosition('player')
	return format(CoordsTextFormat, x * 100, y * 100)
end

local function UpdateZone()
	local pvpType = GetZonePVPInfo()
	if pvpType == 'friendly' then
		text_color = '00ff00'
	elseif pvpType == 'hostile' then
		text_color = 'dd3a00'
	else
		text_color = 'ffff00'
	end
	
	if ShowMainZoneText then
		local zoneText = GetRealZoneText()
		local subZoneText = GetSubZoneText()
		if ShowSubZoneText and (zoneText ~= subZoneText) and (subZoneText ~= '') then
			text_zone = format('%s: %s', zoneText, subZoneText)
		else 
			text_zone = zoneText
		end
	else
		text_zone = GetMinimapZoneText()
	end
end

local function UpdateLDBText()
	obj.text = format('|cff%s%s%s|r', text_color, text_zone, GetCoords())
end

obj.OnClick = function(_, button)
	if IsModifierKeyDown() then
		local subZoneText = GetSubZoneText()
		local zoneText = GetRealZoneText()
		local x, y = GetPlayerMapPosition('player')
		local coords = x~=0 and y~=0 and format(' (%.0f, %.0f)', x * 100, y * 100) or ''
		
		local text = format('%s%s%s', zoneText, (zoneText ~= subZoneText) and (subZoneText ~= '') and format(': %s', subZoneText) or '', coords)
		
		if ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Insert(text)
		else
			ChatFrame_OpenChat(text)
		end
	elseif button == 'LeftButton' then
		ToggleFrame(WorldMapFrame)
	elseif button == 'RightButton' then
		if ATLAS_VERSION then Atlas_Toggle() end
	end
end

obj.OnTooltipShow = function(tooltip)
	if not tooltip or not tooltip.AddLine then return end
	
	local subZoneText = GetSubZoneText()
	local zoneText = GetRealZoneText()
	local x, y = GetPlayerMapPosition('player')
	local coords = x~=0 and y~=0 and format('%.0f, %.0f', x * 100, y * 100)
	
	tooltip:AddLine('|cffff8800leafZone|r')
	tooltip:AddLine(' ')
	
	tooltip:AddDoubleLine(L['Zone'], format('|cff%s%s|r',text_color,zoneText), 1, 1, 0, 1, 1, 1)
	if (zoneText ~= subZoneText) and (subZoneText ~= '') then
		tooltip:AddDoubleLine(L['Subzone'], format('|cff%s%s|r',text_color,subZoneText), 1, 1, 0, 1, 1, 1)
	end
	if coords then
		tooltip:AddDoubleLine(L['Coordinates'], format('|cff%s%s|r',text_color,coords), 1, 1, 0, 1, 1, 1)
	end
	
	tooltip:AddLine(' ')
	tooltip:AddLine(L['|cff00ff00Shift + Click|r insert zone text\ninto chat edit box'],1,1,0)
	tooltip:AddLine(L['|cff00ff00Left-Click|r toggle worldmap'],1,1,0)
	if ATLAS_VERSION then
		tooltip:AddLine(L['|cff00ff00Right-Click|r toggle Atlas'],1,1,0)
	end
end

local function CheckUpdate()
	local inInstance, instanceType = IsInInstance()
	if inInstance and badinstanceType[instanceType] then
		status_ininstance = true
	else
		status_ininstance = false
	end

	if not ShowCoords then return end
	local updating = f:GetScript("OnUpdate")

	if status_ininstance then
		if updating then
			f:SetScript('OnUpdate', nil)
		end
		return
	elseif updating then
		return
	end
	
	elapsed = 0
	f:SetScript('OnUpdate', function(self, el)
		elapsed = elapsed + el
		if (elapsed >= .25) then
			UpdateLDBText()
			elapsed = 0
		end
	end)
end

f:RegisterEvent'PLAYER_ENTERING_WORLD'
if ShowMainZoneText or ShowSubZoneText then
	f:RegisterEvent'ZONE_CHANGED'
	f:RegisterEvent'ZONE_CHANGED_INDOORS'
	f:RegisterEvent'ZONE_CHANGED_NEW_AREA'
	f:RegisterEvent'WORLD_MAP_UPDATE'
end
f:SetScript('OnEvent', function(self, event, ...)
	if event == 'PLAYER_ENTERING_WORLD' then
		CheckUpdate()
	end
	if ShowMainZoneText or ShowSubZoneText then UpdateZone() end
	UpdateLDBText()
end)
